pub struct Lifetime(/* private fields */);Expand description
Abstract representation of a lifetime.
Used to identify a lifetime in LifetimeToken and LifetimeDead.
To differentiate this from a normal Rust lifetime (e.g. 'a), we call the
former a syntactic lifetime, while this is a synthetic lifetime.